package org.wanto.mp3k;

/**
 * Mp3Knife: A mp3 disection utility.
 * Copyright (C) Paul Stanton (wanto_@hotmail.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.util.*;

/**
 * limited api for an mp3 file.
 * this object has limited functionality specific for mp3knife
 * @author: Paul Stanton (wanto_@hotmail.com)
 */
 
public class Mp3
{
	private File file;
	private List frameHeaders; // to be array list of Mp3Headers
	private boolean hasId3v1;

	public Mp3(File _file) throws IOException
	{
		// keep reference to file
		file = _file;

		// test for id3v1 tag at end of file.
		{
			RandomAccessFile raf = new RandomAccessFile(file, "r");
			raf.seek(file.length() - 128);
			byte[] tagStartBytes = new byte[3];
			raf.read(tagStartBytes);
			raf.close();
			if(new String(tagStartBytes).equals("TAG")) hasId3v1 = true;
		}

		// read frame headers
		frameHeaders = read();
	}
	/**
	 * access to file reference
	 */
	public File getFile()
	{
		return file;
	}
	/**
	 * returns data for the frames between the two points including the last point.
	 */
	public byte[] getFrameData(int startFrame, int endFrame) throws IOException
	{
		if(startFrame < 0) startFrame = 0;
		if(endFrame > frameHeaders.size() - 1) endFrame = frameHeaders.size() - 1;
		if(startFrame >= endFrame) return new byte[0];

		Mp3Header firstHeader = (Mp3Header) frameHeaders.get(startFrame);
		Mp3Header lastHeader = (Mp3Header) frameHeaders.get(endFrame);

		int startByte = firstHeader.getPosition();
		int endByte = lastHeader.getPosition()
			+ lastHeader.getFrameSize();

		FileInputStream fis = new FileInputStream(file);
		byte[] bytes = new byte[endByte - startByte];

		fis.skip(startByte);
		fis.read(bytes);
		fis.close();

		return bytes;
	}
/**
 * Returns the headers for all frames in the file.
 * Creation date: (5/05/2002 7:59:22 PM)
 * @return java.util.List
 */
	public List getFrameHeaders()
	{
		return frameHeaders;
	}
	/**
	 * will return the index of the first frame
	 * after the beggining of the specified second
	 * based on an infinate file
	 * when inclusive, will return the index of the frame after the second is reached
	 * Creation date: (29/04/2002 8:47:03 AM)
	 * @param: float
	 * @return: int
	 */
	public int getFrameIndexForSecond(float second, boolean inclusive)
	{
		float result = second / ((Mp3Header) frameHeaders.get(0)).getFrameLength();
		
		if(!inclusive
			|| (inclusive
				&& !(result % 1 == 0)))
		{
			result += 1;
		}
		
		return (int) result;
	}
	/**
	 * returns the bytes that make up a new mp3 file, with the audio data
	 * between the two points and any extra info contained in the file
	 * before and after audio data eg id3v1 and id3v2
	 */
	public List getNewMp3(int startFrame, int endFrame) throws IOException
	{
		List bytes = new LinkedList();

		bytes.add(getPreFrameData());
		bytes.add(getFrameData(startFrame, endFrame));
		bytes.add(getPostFrameData());

		return bytes;
	}
	/**
	 * returns id3v1 tag bytes if any
	 */
	public byte[] getPostFrameData() throws IOException
	{		
		if(!hasId3v1) return new byte[0];
		
		byte[] bytes = new byte[128];

		FileInputStream fis = new FileInputStream(file);
		fis.skip(file.length() - 128);
		fis.read(bytes);
		fis.close();

		return bytes;
	}
	/**
	 * returns any data that came before audio information eg id3v2
	 */
	public byte[] getPreFrameData() throws IOException
	{
		// read from zero to index of first frame.
		byte[] bytes = new byte[((Mp3Header) frameHeaders.get(0)).getPosition()];

		FileInputStream fis = new FileInputStream(file);

		fis.read(bytes);
		fis.close();

		return bytes;
	}
	/**
	 * Will return the number of seconds elapsed when the specified frame is reached
	 * based on an infinate file.
	 * if inclusive, will return the seconds elapsed when the frame is completed
	 * Creation date: (29/04/2002 8:47:03 AM)
	 * @param: int
	 * @return: float
	 */
	public float getSecondForFrameIndex(int frameIndex, boolean inclusive)
	{
		if(!inclusive) frameIndex -= 1;
		return (float) frameIndex * ((Mp3Header) frameHeaders.get(0)).getFrameLength();
	}
	public final static boolean isFrameStart(byte b1, byte b2)
	{
		// test that these two bytes mark the start of a frame header
		return b1 == -1 && (b2 <= -1 && b2 >= -48);
	}
	/**
	 * iterates through and extracts headers for all frames in an mp3
	 */
	private List read() throws IOException
	{
		/*// logging
		long startMilli = System.currentTimeMillis();*/

		FileInputStream fis = new FileInputStream(file);
		List headers = new ArrayList();

		// find and add first header
		Mp3Header lastHeader = readFirstHeader(fis);
		headers.add(lastHeader);

		while(true)
		{
			// initialize read params
			int lastFrameLength = lastHeader.getFrameSize();
			int headerPosition = lastHeader.getPosition() + lastFrameLength;
			byte[] currHeaderBytes = new byte[4];


			try
			{
				// seek to next header position and read
				if(fis.skip(lastFrameLength - 4) != (lastFrameLength - 4))
				{
					// end of file found while skipping
					break;
				}
				if(fis.read(currHeaderBytes) == -1)
				{
					// end of file found while reading
					break;
				}
			}
			catch(IOException e)
			{
				// if crc error: message = Permission denied
				// remove last header as safe guard
				headers.remove(headers.size() - 1);
				break;
			}

			// create header
			try
			{
				lastHeader = new Mp3Header(currHeaderBytes, headerPosition);
			}
			catch(IOException e)
			{
				// invalid header: must be end of file
				break;
			}

			// add to list
			headers.add(lastHeader);
		}

		// done with stream
		fis.close();

		// test validity of last frame, if not complete remove from array
		{
			int eofAudio = ((int) file.length()) - (hasId3v1 ? 128 : 0);
			int supposedEofAudio = lastHeader.getPosition() + lastHeader.getFrameSize();
			if(supposedEofAudio > eofAudio) headers.remove(headers.size() - 1);
		}

		/*// logging
		System.out.println(
			getClass().getName()
			+ ".readHeaders() read "
			+ headers.size()
			+ " headers in "
			+ (System.currentTimeMillis() - startMilli)
			+ " milliseconds");*/

		return headers;
	}
	private Mp3Header readFirstHeader(FileInputStream fis)
	throws IOException
	{
		// find first header
		byte[] currHeaderBytes = new byte[4];

		int index = 0;
		while(true)
		{
			int read = fis.read();
			if(read == -1)
			{
				// eof & no frame marker
				throw new IOException("file was not a valid mp3");
			}

			byte currRead = (byte) read;

			if(isFrameStart(currHeaderBytes[0], currRead))
			{
				currHeaderBytes[1] = currRead;
				break;
			}

			index++;
			currHeaderBytes[0] = currRead;
		}

		// read remainder of first header
		currHeaderBytes[2] = (byte) fis.read();
		currHeaderBytes[3] = (byte) fis.read();

		return new Mp3Header(currHeaderBytes, index - 1);
	}
}
