package org.wanto.mp3k;

/**
 * Mp3Knife: A mp3 disection utility.
 * Copyright (C) Paul Stanton (wanto_@hotmail.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.IOException;
import org.wanto.util.ByteUtils;

/**
 * reads a 4 byte pattern as a mp3 frame header.
 * Provides methods and constants for reading and comparing header values.
 * @author: Paul Stanton (wanto_@hotmail.com)
 */
 
public class Mp3Header
{
	private byte[] bytes = new byte[4];
	private int position = -1;

	public static final short NOT_SET = -1;

	public static final short MPEG_1 = 1; // (ISO/IEC 11172-3)
	public static final short MPEG_2 = 2; // (ISO/IEC 13818-3)
	public static final short MPEG_2_5 = 0;

	private static final short MPEG_1_SPF = 1152;
	private static final short MPEG_2_SPF = 576;

	public static final short LAYER_I = 1;
	public static final short LAYER_II = 2;
	public static final short LAYER_III = 3;

	public static final short CHANNEL_STEREO = 0;
	public static final short CHANNEL_JOINT_STERIO = 1;
	public static final short CHANNEL_DUAL = 2;
	public static final short CHANNEL_SINGLE = 3;

	public static final short EMPASIS_NONE = 0;
	public static final short EMPASIS_50_15_MS = 1;
	public static final short EMPASIS_CCIT_J_17 = 3;

	public static final short MODEEXT_BANDS_4_31 = 0;
	public static final short MODEEXT_BANDS_8_31 = 1;
	public static final short MODEEXT_BANDS_12_31 = 2;
	public static final short MODEEXT_BANDS_16_31 = 3;

	public static final short MODEEXT_0MS_0IS = 4;
	public static final short MODEEXT_MS_0IS = 5;
	public static final short MODEEXT_0MS_IS = 6;
	public static final short MODEEXT_MS_IS = 7;

	public Mp3Header(byte[] _bytes, int _position) throws IOException
	{
		// validate
		if(!Mp3.isFrameStart(_bytes[0], _bytes[1]))
		{
			throw new IOException(
				"not valid header : "
				+ ByteUtils.toString(_bytes[0]) + " "
				+ ByteUtils.toString(_bytes[1]) + " "
				+ ByteUtils.toString(_bytes[2]) + " "
				+ ByteUtils.toString(_bytes[3]));
		}

		// record position
		position = _position;

		// record data info
		bytes[0] = _bytes[0];
		bytes[1] = _bytes[1];
		bytes[2] = _bytes[2];
		bytes[3] = _bytes[3];
	}
	public int getBitrate()
	{
		int index = 0;
		if(ByteUtils.isBitSet(bytes[2], 1)) index += 8;
		if(ByteUtils.isBitSet(bytes[2], 2)) index += 4;
		if(ByteUtils.isBitSet(bytes[2], 3)) index += 2;
		if(ByteUtils.isBitSet(bytes[2], 4)) index += 1;

		int index2 = 0;
		{
			int version = getVersion();
			int layer = getLayer();

			if(version == MPEG_1)
			{
				if(layer == LAYER_I) index2 = 0;
				else if(layer == LAYER_II) index2 = 1;
				else if(layer == LAYER_III) index2 = 2;
			}
			else if(version == MPEG_2)
			{
				if(layer == LAYER_I) index2 = 3;
				else if(layer == LAYER_II || layer == LAYER_III) index2 = 4;
			}
		}

		int[][] bitrates = new int[][]
		{
			{0,	 32,  64,  96,  128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 999}, // V1L1
			{0,	 32,  48,  56,  64,  80,  96,  112, 128, 160, 192, 224, 256, 320, 384, 999}, // V1L2
			{0,	 32,  40,  48,  56,  64,  80,  96,  112, 128, 160, 192, 224, 256, 320, 999}, // V1L3
			{0,	 32,  48,  56,  64,  80,  96,  112, 128, 144, 160, 176, 192, 224, 256, 999}, // V2L1
			{0,	 8,   16,  24,  32,  40,  48,  56,  64,  80,  96,  112, 128, 144, 160, 999}  // V2L1L2
		};

		return bitrates[index2][index];
	}
	// access to byte data
	public byte[] getBytes()
	{
		return bytes;
	}
	public short getChannel()
	{
		short result = 0;

		if(ByteUtils.isBitSet(bytes[3], 1)) result += 2;
		if(ByteUtils.isBitSet(bytes[3], 2)) result += 1;

		return result;
	}
	public short getEmpasis()
	{
		short result = 0;

		if(ByteUtils.isBitSet(bytes[3], 1)) result += 2;
		if(ByteUtils.isBitSet(bytes[3], 2)) result += 1;

		if(result == 2) result = NOT_SET;
		return result;
	}
	/**
	 * @return:float : frame length in seconds
	 */
	public float getFrameLength()
	{
		return (float) ((getVersion() == MPEG_1) ? MPEG_1_SPF : MPEG_2_SPF) / (float) getSampleRateFreq();
	}
	/**
	 * calculates the length in bytes of the current frame.
	 * rely's on the version, layer sampleRateFreq & padding of the frame
	 * Internally, the coefficient value should correspond with the following rule:
	 * V1, L1 = 48; V1, L2/3 = 144; V2, L1 = 24; V2, L2/3 = 72;
	 */
	public int getFrameSize()
	{
		int x = 24;
		if(getVersion() == MPEG_1) x *= 2;
		if(getLayer() != LAYER_I) x *= 3;

		return ((x * (getBitrate() * 1000)) / getSampleRateFreq())
			+ getPadding();
//			+ (isProtected() ? 2 : 0);
	}
	public short getLayer()
	{
		if(ByteUtils.isBitSet(bytes[1], 6))
		{
			if(ByteUtils.isBitSet(bytes[1], 7)) return LAYER_I;
			return LAYER_II;
		}
		if(ByteUtils.isBitSet(bytes[1], 7)) return LAYER_III;
		return NOT_SET;
	}
	public short getModeExt()
	{
		short channel = getChannel();

		if(channel != CHANNEL_JOINT_STERIO) return NOT_SET;

		short layer = getLayer();
		boolean bitA = ByteUtils.isBitSet(bytes[3], 3);
		boolean bitB = ByteUtils.isBitSet(bytes[3], 4);

		if(layer == LAYER_I || layer == LAYER_II)
		{
			if(bitA)
			{
				if(bitB) return MODEEXT_BANDS_16_31;
				return MODEEXT_BANDS_12_31;
			}
			else
			{
				if(bitB) return MODEEXT_BANDS_8_31;
				return MODEEXT_BANDS_4_31;
			}
		}
		else if(layer == LAYER_III)
		{
			if(bitA)
			{
				if(bitB) return MODEEXT_MS_IS;
				return MODEEXT_MS_0IS;
			}
			else
			{
				if(bitB) return MODEEXT_0MS_IS;
				return MODEEXT_0MS_0IS;
			}
		}
		return NOT_SET;
	}
	public int getPadding()
	{
		int padding = isPadded() ? 1 : 0;
		if(getLayer() == LAYER_I) padding *= 4;
		return padding;
	}
	// access to position (in bytes) of frame in file
	public int getPosition()
	{
		return position;
	}
	public int getSampleRateFreq()
	{
		int freqIndex = -1;
		int version = getVersion();
		boolean bitA = ByteUtils.isBitSet(bytes[2], 5);
		boolean bitB = ByteUtils.isBitSet(bytes[2], 6);

		if(bitA)
		{
			if(!bitB) freqIndex = 2;
		}
		else
		{
			if(bitB) freqIndex = 1;
			else freqIndex = 0;
		}

		int[][] freqs = new int[][]
		{
			{11025, 12000, 8000},  // version 2.5
			{44100, 48000, 32000}, // version 1
			{22050, 24000, 16000}  // version 2
		};

		if(version < 0 || version > 2 || freqIndex == -1) return NOT_SET;
		return freqs[version][freqIndex];
	}
	/**
	 * determines the mpeg version of the frame
	 */
	public short getVersion()
	{
		boolean bitA = ByteUtils.isBitSet(bytes[1], 4);
		boolean bitB = ByteUtils.isBitSet(bytes[1], 5);

		if(bitA)
		{
			if(bitB) return MPEG_1;
			return MPEG_2;
		}
		if(!bitB) return MPEG_2_5;

		return NOT_SET;
	}
	public boolean isCopyright()
	{
		return ByteUtils.isBitSet(bytes[3], 5);
	}
	public boolean isOriginal()
	{
		return ByteUtils.isBitSet(bytes[3], 6);
	}
	public boolean isPadded()
	{
		return ByteUtils.isBitSet(bytes[2], 7);
	}
	public boolean isPrivateBitSet()
	{
		return ByteUtils.isBitSet(bytes[2], 8);
	}
	public boolean isProtected()
	{
		return !ByteUtils.isBitSet(bytes[1], 8);
	}
	public String toString()
	{
		StringBuffer output = new StringBuffer();

		output.append(getClass().getName());
		output.append(":");
		output.append(ByteUtils.toString(bytes[0]));
		output.append(" ");
		output.append(ByteUtils.toString(bytes[1]));
		output.append(" ");
		output.append(ByteUtils.toString(bytes[2]));
		output.append(" ");
		output.append(ByteUtils.toString(bytes[3]));
		output.append(" ");
		output.append(position);

		return output.toString();
	}
}
