package org.wanto.mp3k.ui.swing;

/**
 * Mp3Knife: A mp3 disection utility.
 * Copyright (C) Paul Stanton (wanto_@hotmail.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import org.wanto.mp3k.*;
import org.wanto.util.*;
import org.wanto.util.swing.*;

import java.util.*;
import java.io.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Swing interface for mp3knife application.
 * @author: Paul Stanton (wanto_@hotmail.com)
 */

public class Application extends JFrame
{
	private final static int MENU_OPEN = 0;
	private final static int MENU_CLOSE = 1;
	private final static int MENU_APPEND = 3;
	private final static int MENU_LICENSE_COPY = 4;
	private final static int MENU_LICENSE_WARRANTY = 5;
	private final static int MENU_LICENSE_WHY = 6;
	private final static int MENU_VIEW_FULL = 7;
	private final static int MENU_VIEW_COMPACT = 8;

	private final static String ACCEPT_FILE_EXT = ".mp3";

	private Mp3 mp3;

	private final JTextField startSecond = new RestrictiveTextField("1234567890:.");
	private final JTextField endSecond = new RestrictiveTextField("1234567890:.");
	private final JTextField startFrame = new RestrictiveTextField("1234567890");
	private final JTextField endFrame = new RestrictiveTextField("1234567890");
	private final JTextField status = new JTextField(getMessage("status.initialising"));

	private final JLabel startSecondLabel = new JLabel(getMessage("field.start"));
	private final JLabel endSecondLabel = new JLabel(getMessage("field.end"));
	private final JLabel startFrameLabel = new JLabel(getMessage("field.start"));
	private final JLabel endFrameLabel = new JLabel(getMessage("field.end"));
	private final JLabel frameLabel = new JLabel(getMessage("field.frame"));
	private final JLabel secondLabel = new JLabel(getMessage("field.second"));

	private final JButton createButton = new JButton(getMessage("button.createFile"));
	private final JButton removeButton = new JButton(getMessage("button.removeFile"));

	private final Mp3InfoPane mp3InfoPane = new Mp3InfoPane(getLocale(), getMessage("info.default"));

	private class MenuDispatcher implements ActionListener
	{
		private int optionCode = -1;

		public void actionPerformed(ActionEvent evt)
		{
			switch(optionCode)
			{
				case MENU_OPEN:
					openFile();
					break;

				case MENU_CLOSE:
					closeFile();
					break;

				case MENU_APPEND:
					createJoinedFile();
					break;

				case MENU_LICENSE_COPY:
					alert(getMessage("license.title.copy"), "Copyright (C) Paul Stanton (wanto_@hotmail.com)\n\nThis program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.");
					break;

				case MENU_LICENSE_WARRANTY:
					alert(getMessage("license.title.warranty"), "This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.");
					break;

				case MENU_LICENSE_WHY:
					alert(getMessage("license.title.why"), getMessage("license.why"));
					break;

				case MENU_VIEW_FULL:
					mp3InfoPane.setVisible(true);
					pack();
					break;

				case MENU_VIEW_COMPACT:
					mp3InfoPane.setVisible(false);
					pack();
					break;

				default:
					System.out.println(getClass().getName() + "; not implemented:"+ optionCode);
					break;
			}
		}

		MenuDispatcher(JFrame frame, int _optionCode)
		{
			optionCode = _optionCode;
		}
	}


	/**
	 * constructs a new Mp3Knife Application window.
	 * attempts to use the first command line argument as a file reference,
	 * otherwise, initialises with no file.
	 * Creation date: (29/04/2002 8:06:59 AM)
	 * @param: String[]
	 */
	public Application(String[] args)
	{
		// initialise window
		super();

		setTitle(getMessage("window.title"));

		// attempt to use args.
		if (args.length > 0 && args[0] != null)
		{
			File file = new File(args[0]);
			if (file.exists())
			{
				try
				{
					mp3 = new Mp3(file);
				}
				catch (IOException e)
				{
					// continue without file
				}
			}
		}

		init();
	}
	private void alert(String title, String message)
	{
		JDialog notification = new JDialog(this, title, true);

		JTextPane tpane = new JTextPane();
		tpane.setText(message);
		tpane.setOpaque(false);
		tpane.setEnabled(false);

		JScrollPane scrollPane = new JScrollPane(tpane);
		scrollPane.setBorder(new javax.swing.border.EmptyBorder(2, 2, 2, 2));
		scrollPane.setPreferredSize(new Dimension(300, 200));

		notification.getContentPane().add(scrollPane);

		notification.pack();
		notification.setVisible(true);
	}
	private void closeFile()
	{
		mp3 = null;
		setTitle(getMessage("window.title"));

		// suggest garbage collection
		System.gc();
		disableOptions();
		refreshInfo();
	}
	private void createFileExcludeSlice(int sliceStart, int sliceEnd)
	{
		status.setText(getMessage("status.creatingNewFile"));

		byte[] preData = null;
		byte[] audioPreSlice = null;
		byte[] audioPostSlice = null;
		byte[] postData = null;

		try
		{
			preData = mp3.getPreFrameData();
			postData = mp3.getPostFrameData();
			audioPreSlice = mp3.getFrameData(0, sliceStart);
			audioPostSlice = mp3.getFrameData(sliceEnd, mp3.getFrameHeaders().size() - 1);
		}
		catch(FileNotFoundException e)
		{
			alert(getMessage("alert.title.sourceFileNotFound"), getMessage("alert.sourceFileNotFound"));
			status.setText(getMessage("status.failed"));
			return;
		}
		catch(IOException e)
		{
			alert(getMessage("alert.title.failedSourceFileRead"), getMessage("alert.failedSourceFileRead"));
			status.setText(getMessage("status.failed"));
			return;
		}

		// stuff byte arrays into one
		java.util.List newFileBytes = new LinkedList();
		newFileBytes.add(preData);
		newFileBytes.add(audioPreSlice);
		newFileBytes.add(audioPostSlice);
		newFileBytes.add(postData);

		// save complete byte array
		if(!saveBytesToFile(newFileBytes, null))
		{
			status.setText(getMessage("status.failed"));
			return;
		}

		status.setText(getMessage("status.ready"));
	}
	private void createJoinedFile()
	{
		status.setText(getMessage("status.creatingNewFile"));

		// get 2nd source file
		Mp3 newMp3 = openMp3();

		if(!validateFileCombination(mp3, newMp3))
		{
			status.setText(getMessage("status.cancelled"));
			return;
		}

		// choose which file to use as source for extra data (id3)
		ConfirmDialog id3Opt = new ConfirmDialog(
			this,
			getMessage("confirm.id3choose.title"),
			getMessage("confirm.id3choose", new String[]{mp3.getFile().getName(), newMp3.getFile().getName()}),
			getMessage("confirm.ok"),
			getMessage("confirm.cancel"));
		id3Opt.setOptionTexts(getMessage("confirm.id3choose.a"), getMessage("confirm.id3choose.b"));
		id3Opt.show();

		byte[] preData = null;
		byte[] mp3Audio = null;
		byte[] newMp3Audio = null;
		byte[] postData = null;

		try
		{
			if(id3Opt.getValue())
			{
				preData = mp3.getPreFrameData();
				postData = mp3.getPostFrameData();
			}
			else
			{
				preData = newMp3.getPreFrameData();
				postData = newMp3.getPostFrameData();
			}

			mp3Audio = mp3.getFrameData(0, mp3.getFrameHeaders().size() - 1);
			newMp3Audio = newMp3.getFrameData(0, newMp3.getFrameHeaders().size() - 1);
		}
		catch(FileNotFoundException e)
		{
			alert(getMessage("alert.title.sourceFileNotFound"), getMessage("alert.sourceFileNotFound"));
			status.setText(getMessage("status.failed"));
			return;
		}
		catch(IOException e)
		{
			alert(getMessage("alert.title.failedSourceFileRead"), getMessage("alert.failedSourceFileRead"));
			status.setText(getMessage("status.failed"));
			return;
		}

		// stuff byte arrays into one
		java.util.List newFileBytes = new java.util.LinkedList();
		newFileBytes.add(preData);
		newFileBytes.add(mp3Audio);
		newFileBytes.add(newMp3Audio);
		newFileBytes.add(postData);

		// save complete byte array
		if(!saveBytesToFile(newFileBytes, null))
		{
			status.setText(getMessage("status.failed"));
			return;
		}

		status.setText(getMessage("status.ready"));
	}
	private void createSplitFile(int startFrameIndex, int endFrameIndex)
	{
		status.setText(getMessage("status.creatingNewFile"));

		java.util.List newFileBytes = null;
		try
		{
			newFileBytes = mp3.getNewMp3(startFrameIndex, endFrameIndex);
		}
		catch(FileNotFoundException e)
		{
			alert(getMessage("alert.title.sourceFileNotFound"), getMessage("alert.sourceFileNotFound"));
			status.setText(getMessage("status.failed"));
			return;
		}
		catch(IOException e)
		{
			alert(getMessage("alert.title.failedSourceFileRead"), getMessage("alert.failedSourceFileRead"));
			status.setText(getMessage("status.failed"));
			return;
		}

		if(!saveBytesToFile(newFileBytes, null))
		{
			status.setText(getMessage("status.failed"));
		}

		status.setText(getMessage("status.ready"));
	}
	private void disableOptions()
	{
		startFrame.setEnabled(false);
		endFrame.setEnabled(false);
		startSecond.setEnabled(false);
		endSecond.setEnabled(false);

		startFrameLabel.setForeground(Color.gray);
		endFrameLabel.setForeground(Color.gray);
		frameLabel.setForeground(Color.gray);

		startSecondLabel.setForeground(Color.gray);
		endSecondLabel.setForeground(Color.gray);
		secondLabel.setForeground(Color.gray);

		createButton.setEnabled(false);
		removeButton.setEnabled(false);

		getJMenuBar().getMenu(0).getItem(1).setEnabled(false);
		getJMenuBar().getMenu(0).getItem(2).setEnabled(false);
		
		refreshInfo();
		pack();

		status.setText(getMessage("status.selectFileOpen"));
	}
	private void enableOptions()
	{
		startFrame.setEnabled(true);
		endFrame.setEnabled(true);
		startSecond.setEnabled(true);
		endSecond.setEnabled(true);

		startFrameLabel.setForeground(Color.black);
		endFrameLabel.setForeground(Color.black);
		frameLabel.setForeground(Color.black);

		startSecondLabel.setForeground(Color.black);
		endSecondLabel.setForeground(Color.black);
		secondLabel.setForeground(Color.black);

		createButton.setEnabled(true);
		removeButton.setEnabled(true);

		getJMenuBar().getMenu(0).getItem(1).setEnabled(true);
		getJMenuBar().getMenu(0).getItem(2).setEnabled(true);

		refreshInfo();
		pack();

		status.setText(getMessage("status.ready"));
	}
	/**
	 * generates the menubar for the application.
	 */
	protected JMenuBar getAppMenuBar()
	{
		JMenuBar menuBar = new JMenuBar();
		JMenuItem currMenuItem;
		JMenu currMenu;

		currMenu = new JMenu(getMessage("menu.file"));
		currMenu.setMnemonic('F');

		currMenu.add(currMenuItem = new JMenuItem(getMessage("menu.file.open")));
		currMenuItem.setMnemonic('O');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_OPEN));
		currMenu.add(currMenuItem = new JMenuItem(getMessage("menu.file.append")));
		currMenuItem.setMnemonic('A');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_APPEND));
		currMenu.add(currMenuItem = new JMenuItem(getMessage("menu.file.close")));
		currMenuItem.setMnemonic('C');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_CLOSE));

		menuBar.add(currMenu);

		currMenu = new JMenu(getMessage("menu.license"));
		currMenu.setMnemonic('L');

		currMenu.add(currMenuItem = new JMenuItem(getMessage("license.title.copy")));
		currMenuItem.setMnemonic('P');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_LICENSE_COPY));
		currMenu.add(currMenuItem = new JMenuItem(getMessage("license.title.warranty")));
		currMenuItem.setMnemonic('W');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_LICENSE_WARRANTY));
		currMenu.add(currMenuItem = new JMenuItem(getMessage("license.title.why")));
		currMenuItem.setMnemonic('H');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_LICENSE_WHY));

		menuBar.add(currMenu);

		currMenu = new JMenu(getMessage("menu.view"));
		currMenu.setMnemonic('V');
		ButtonGroup viewGroup = new ButtonGroup();

		currMenu.add(currMenuItem = new JRadioButtonMenuItem(getMessage("menu.view.full")));
		viewGroup.add(currMenuItem);
		currMenuItem.setMnemonic('U');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_VIEW_FULL));
		currMenu.add(currMenuItem = new JRadioButtonMenuItem(getMessage("menu.view.compact")));
		currMenuItem.setSelected(true);
		viewGroup.add(currMenuItem);
		currMenuItem.setMnemonic('M');
		currMenuItem.addActionListener(new MenuDispatcher(this, MENU_VIEW_COMPACT));

		menuBar.add(currMenu);

		return menuBar;
	}
	/**
	 * constructs the main panel for the application. The JPanel returned contains
	 * two textfields and a button for entry of the parameters to contruct
	 * a new file.
	 * Creation date: (29/04/2002 11:05:12 PM)
	 * @return: JPanel
	 */
	private JPanel getButtonPanel()
	{
		createButton.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{
					try
					{
						int startF = Integer.parseInt(startFrame.getText());
						int endF = Integer.parseInt(endFrame.getText());

						if(startF > endF) throw new NumberFormatException();
						createSplitFile(startF, endF);
					}
					catch(NumberFormatException e)
					{
						alert(getMessage("alert.title.invalidParamEntry"), getMessage("alert.invalidParamEntry"));
						startFrame.setText(null);
						endFrame.setText(null);
					}
				}
			}
		);
		removeButton.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent event)
				{
					try
					{
						int startF = Integer.parseInt(startFrame.getText());
						int endF = Integer.parseInt(endFrame.getText());

						if(startF > endF) throw new NumberFormatException();
						createFileExcludeSlice(startF, endF);
					}
					catch(NumberFormatException e)
					{
						alert(getMessage("alert.title.invalidParamEntry"), getMessage("alert.invalidParamEntry"));
						startFrame.setText(null);
						endFrame.setText(null);
					}
				}
			}
		);
		
		JPanel buttons = new JPanel(new GridLayout(1,2));
		buttons.add(createButton);
		buttons.add(removeButton);
		
		return buttons;
	}
	/**
	 * constructs the main panel for the application. The JPanel returned contains
	 * two textfields and a button for entry of the parameters to contruct
	 * a new file.
	 * Creation date: (29/04/2002 11:05:12 PM)
	 * @return: JPanel
	 */
	private JPanel getMainPanel()
	{
		JPanel inputs = new JPanel(new GridBagLayout());

		initFieldEventHandlers();		

		// set colums equal so the GridBagLayout they are
		// eventually added to (mainPanel())
		// can set their weight properly
		startFrame.setColumns(1);
		startSecond.setColumns(1);
		endFrame.setColumns(1);
		endSecond.setColumns(1);
		
		// set defaults for layout constraints
		GridBagConstraints c = new GridBagConstraints();
		c.insets.top = 2;
		c.insets.right = 2;
		c.insets.left = 2;
		
		// 1st row
		c.gridy = 0;
		
		c.gridx = 0;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		inputs.add(frameLabel, c);
		
		c.gridx = 1;
		c.anchor = GridBagConstraints.EAST;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		inputs.add(startFrameLabel, c);
		
		c.gridx = 2;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		inputs.add(startFrame, c);
		
		c.gridx = 3;
		c.anchor = GridBagConstraints.EAST;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		inputs.add(endFrameLabel, c);
		
		c.gridx = 4;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		inputs.add(endFrame, c);
		
		// 2nd row
		c.gridy = 1;
		
		c.gridx = 0;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		inputs.add(secondLabel, c);
		
		c.gridx = 1;
		c.anchor = GridBagConstraints.EAST;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		inputs.add(startSecondLabel, c);
		
		c.gridx = 2;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		inputs.add(startSecond, c);
		
		c.gridx = 3;
		c.anchor = GridBagConstraints.EAST;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		inputs.add(endSecondLabel, c);
		
		c.gridx = 4;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		inputs.add(endSecond, c);

		// 3rd row
		c.gridy = 2;
		
		c.gridx = 0;
		c.gridwidth = 5;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		inputs.add(getButtonPanel(), c);

		// 4th row
		c.gridy = 3;

		c.gridx = 0;
		c.gridwidth = 5;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;
		c.weighty = 1;
		inputs.add(mp3InfoPane, c);
		
		return inputs;
	}
	protected String getMessage(String key)
	{
		ResourceBundle words = null;
		try
		{
			words = ResourceBundle.getBundle("i18n", getLocale());
			return words.getString(key);
		}
		catch(MissingResourceException e)
		{
			// no i18n file, return the key.
			return key;
		}
	}
	protected String getMessage(String key, String[] args)
	{
		// retrieve message
		String message = getMessage(key);
		if(args == null) return message;

		// replace args
		for(int i = 0; i < args.length; i++)
		{
			message = StringUtils.replace(message, "{" + i + "}", args[i]);
		}

		// return complete message
		return message;
	}
	private void init()
	{
		//setResizable(false);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(getMainPanel(), BorderLayout.CENTER);

		status.setOpaque(false);
		status.setEditable(false);
		status.setFont(new Font("sansserif", Font.BOLD, status.getFont().getSize()));
		status.setForeground(Color.black);
		getContentPane().add(status, BorderLayout.SOUTH);

		setJMenuBar(getAppMenuBar());

		if(mp3 == null) disableOptions();
		else enableOptions();
		
		refreshInfo();
	}
	/**
	 * sets up the special features of the input fields.
	 * Creation date: (29/04/2002 11:05:12 PM)
	 */
	private void initFieldEventHandlers()
	{
		initFrameKeyEventHandlers();
		initSecondFocusEventHandlers();
		initSecondKeyEventHandlers();
	}
	/**
	 * sets up the special features of the input fields.
	 * Creation date: (29/04/2002 11:05:12 PM)
	 */
	private void initFrameKeyEventHandlers()
	{
		startFrame.addKeyListener
		(
			new KeyListener()
			{
				public void keyPressed(KeyEvent evt){}
				public void keyReleased(KeyEvent evt)
				{
					try
					{
						startSecond.setText(
							TimeUtils.getTime(
								mp3.getSecondForFrameIndex(
									Integer.parseInt(
										startFrame.getText()), false)));
					}
					catch(NumberFormatException e)
					{
						startSecond.setText("");
					}
					
					startSecond.setCaretPosition(0);
					startSecond.setScrollOffset(0);
				}
				public void keyTyped(KeyEvent evt){}
			}
		);
		endFrame.addKeyListener
		(
			new KeyListener()
			{
				public void keyPressed(KeyEvent evt){}
				public void keyReleased(KeyEvent evt)
				{
					try
					{
						endSecond.setText(
							TimeUtils.getTime(
								mp3.getSecondForFrameIndex(
									Integer.parseInt(
										endFrame.getText()), true)));
					}
					catch(NumberFormatException e)
					{
						endSecond.setText("");
					}
					
					endSecond.setCaretPosition(0);
					endSecond.setScrollOffset(0);
				}
				public void keyTyped(KeyEvent evt){}
			}
		);
	}
	/**
	 * sets up the special features of the input fields.
	 * Creation date: (29/04/2002 11:05:12 PM)
	 */
	private void initSecondFocusEventHandlers()
	{
		startSecond.addFocusListener
		(
			new FocusListener()
			{
				public void focusGained(FocusEvent evt){}
				public void focusLost(FocusEvent evt)
				{
					try
					{
						startSecond.setText(
							TimeUtils.getTime(
								mp3.getSecondForFrameIndex(
									Integer.parseInt(
										startFrame.getText()), false)));
					}
					catch(NumberFormatException e)
					{
						endSecond.setText("");
					}
					
					startSecond.setCaretPosition(0);
					startSecond.setScrollOffset(0);
				}
			}
		);
		endSecond.addFocusListener
		(
			new FocusListener()
			{
				public void focusGained(FocusEvent evt){}
				public void focusLost(FocusEvent evt)
				{
					try
					{
						endSecond.setText(
							TimeUtils.getTime(
								mp3.getSecondForFrameIndex(
									Integer.parseInt(
										endFrame.getText()), true)));
					}
					catch(NumberFormatException e)
					{
						endSecond.setText("");
					}
					
					endSecond.setCaretPosition(0);
					endSecond.setScrollOffset(0);
				}
			}
		);
	}
	/**
	 * sets up the special features of the input fields.
	 * Creation date: (29/04/2002 11:05:12 PM)
	 */
	private void initSecondKeyEventHandlers()
	{
		startSecond.addKeyListener
		(
			new KeyListener()
			{
				public void keyPressed(KeyEvent evt){}
				public void keyReleased(KeyEvent evt)
				{
					startFrame.setText(
						String.valueOf(
							mp3.getFrameIndexForSecond(
								TimeUtils.getSeconds(
									startSecond.getText()), false)));
						
					startFrame.setCaretPosition(0);
					startFrame.setScrollOffset(0);
				}
				public void keyTyped(KeyEvent evt){}
			}
		);
		endSecond.addKeyListener
		(
			new KeyListener()
			{
				public void keyPressed(KeyEvent evt){}
				public void keyReleased(KeyEvent evt)
				{
					endFrame.setText(
						String.valueOf(
							mp3.getFrameIndexForSecond(
								TimeUtils.getSeconds(
									endSecond.getText()), true)));
						
					endFrame.setCaretPosition(0);
					endFrame.setScrollOffset(0);
				}
				public void keyTyped(KeyEvent evt){}
			}
		);
	}
	/**
	 * runnable main for applicaiton. will load a new instance of Mp3Knife.
	 * Creation date: (29/04/2002 8:09:28 AM)
	 * @param: String[]
	 */
	public static void main(String[] args)
	{
		// create Application
		JFrame frame = new Application(args);

		// register close action
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});

		// show application
		frame.pack();
		frame.setVisible(true);
	}
	private void openFile()
	{
		status.setText(getMessage("status.opening"));

		mp3 = openMp3();
		if (mp3 != null)
		{
			setTitle(getMessage("window.title") + " {" + mp3.getFile().getName() + "}");
			enableOptions();
		}
		else
		{
			disableOptions();
		}

		refreshInfo();
	}
	private Mp3 openMp3()
	{
		Mp3 newMp3 = null;

		boolean goWithFile = true;
		String fileName = null;
		do
		{
			goWithFile = true;

			FileDialog fd = new FileDialog(this, getMessage("window.openfile"), FileDialog.LOAD);
			fd.setFilenameFilter(new ExtensionFilenameFilter(ACCEPT_FILE_EXT));
			fd.show();

			// repaints the window. the open dialog may have corrupted the ui.
			paintAll(getGraphics());

			if(fd.getFile() == null)
			{
				return null;
			}

			if(!fd.getFile().toLowerCase().trim().endsWith(ACCEPT_FILE_EXT))
			{
				ConfirmDialog conf = new ConfirmDialog(
					this,
					getMessage("confirm.title.wrongFileExt"),
					getMessage("confirm.wrongFileExt"),
					getMessage("confirm.ok"),
					getMessage("confirm.cancel"));
				conf.setOptionTexts(getMessage("confirm.wrongFileExt.true"), getMessage("confirm.wrongFileExt.false"));
				conf.show();
				goWithFile = conf.getValue();
			}
			fileName = fd.getDirectory() + fd.getFile();
		}
		while(!goWithFile);

		try
		{
			newMp3 = new Mp3(new File(fileName));
		}
		catch(IOException e)
		{
			alert(getMessage("alert.title.fileInvalid"), getMessage("alert.fileInvalid"));
			return null;
		}

		return newMp3;
	}
	/**
	 * updates the content of the mp3 info pane.
	 */
	private void refreshInfo()
	{
		StringBuffer sb = new StringBuffer();

		try
		{
			// get string representation of mp3
			Mp3Header firstHeader = (Mp3Header) mp3.getFrameHeaders().get(0);
			
			sb.append(mp3.getFile().getPath());
			sb.append("\n");
			sb.append(mp3.getFile().length());
			sb.append(" ");
			sb.append(getMessage("mp3value.length.bytes"));
			sb.append(", ");
			sb.append(mp3.getFrameHeaders().size());
			sb.append(" ");
			sb.append(getMessage("mp3value.length.frames"));
			sb.append(", ");
			sb.append(mp3.getSecondForFrameIndex(mp3.getFrameHeaders().size(), true));
			sb.append(" ");
			sb.append(getMessage("mp3value.length.seconds"));

			sb.append("\n");
			sb.append(getMessage("mp3value.version.prefix"));
			sb.append(firstHeader.getVersion());

			sb.append(", ");
			sb.append(getMessage("mp3value.layer"));
			sb.append(firstHeader.getLayer());

			sb.append(", ");
			sb.append(firstHeader.getBitrate());
			sb.append(getMessage("mp3value.bitratesuffix"));

			sb.append(", ");
			sb.append(firstHeader.getSampleRateFreq());
			sb.append(getMessage("mp3value.sampleratefreqsuffix"));

			sb.append(", ");
			sb.append(getMessage("mp3value.channel." + firstHeader.getChannel()));

			sb.append(firstHeader.isProtected() ? ", " + getMessage("mp3value.crcs") : "");
		}
		catch(NullPointerException e)
		{
			sb = new StringBuffer(getMessage("info.default"));
		}

		mp3InfoPane.getTextPane().setText(sb.toString());
	}
	private boolean saveBytesToFile(java.util.List fileBytes, File file)
	{
		if(file == null)
		{
			FileDialog fd = new FileDialog(this, getMessage("window.newFileName"), FileDialog.SAVE);
			fd.setDirectory(mp3.getFile().getParent());
			fd.setFilenameFilter(new ExtensionFilenameFilter(ACCEPT_FILE_EXT));
			fd.show();

			if(fd.getFile() == null) return false;

			file = new File(fd.getDirectory() + fd.getFile());
		}

		try
		{
			FileOutputStream fos = new FileOutputStream(file);
			for(int i = 0; i < fileBytes.size(); i++)
			{
				fos.write((byte[]) fileBytes.get(i));
			}
			fos.close();
		}
		catch(IOException e)
		{
			alert(getMessage("alert.title.failedFileCreate"), getMessage("alert.failedFileCreate"));
			return false;
		}

		return true;
	}
	private boolean saveBytesToFile(Mp3FileBytes fileBytes, File file)
	{
		if(file == null)
		{
			FileDialog fd = new FileDialog(this, getMessage("window.newFileName"), FileDialog.SAVE);
			fd.setDirectory(mp3.getFile().getParent());
			fd.setFilenameFilter(new ExtensionFilenameFilter(ACCEPT_FILE_EXT));
			fd.show();

			if(fd.getFile() == null) return false;

			file = new File(fd.getDirectory() + fd.getFile());
		}

		try
		{
			FileOutputStream fos = new FileOutputStream(file);
			fos.write(fileBytes.getPreFrameData());
			fos.write(fileBytes.getFrameData());
			fos.write(fileBytes.getPostFrameData());
			fos.close();
		}
		catch(IOException e)
		{
			alert(getMessage("alert.title.failedFileCreate"), getMessage("alert.failedFileCreate"));
			return false;
		}

		return true;
	}
	/**
	 * returns true if the system, or the user is happy with the compatablility of the two files.
	 */
	private boolean validateFileCombination(Mp3 file1, Mp3 file2)
	{
		// prelim validation
		if(file1 == null || file2 == null) return false;

		// cast
		Mp3Header header1 = (Mp3Header) file1.getFrameHeaders().get(0);
		Mp3Header header2 = (Mp3Header) file2.getFrameHeaders().get(0);

		// discrepency tests....
		boolean need2ask = false;
		if(header1.getChannel() != header2.getChannel()) need2ask = true;
		if(header1.getVersion() != header2.getVersion()) need2ask = true;
		if(header1.getLayer() != header2.getLayer()) need2ask = true;

		// user notification
		if(need2ask)
		{
			ConfirmDialog conf = new ConfirmDialog(
				this,
				getMessage("confirm.filecombine.title"),
				getMessage("confirm.filecombine"),
				getMessage("confirm.ok"),
				getMessage("confirm.cancel"));
			conf.show();

			if(!conf.getValue()) return false;
		}

		return true;
	}
}
